/*
OPERATORS
and             &&
and_eq          &=
bitand          &
bitor           |
not             !
not_eq          !=
or              ||
or_eq           |=
xor             ^
xor_eq          ^=

PWM
https://www.baldengineer.com/millis-ind-on-off-times.html

Eltek charger status report
ID 0x3X5 8bytes
  B0 = Charger status    (1-idle, 2-charge, 3-error, 4-errorx)
  B1 = AC-Mains current LSB (hex/10 A)
  B2 = AC-Mains current HSB (hex/10 A)
  B3 = DC-charge current LSB (hex/10 A)
  B4 = DC-charge current HSB (hex/10 A)
  B5 = DC-charge voltage LSB (hex/10 V)
  B6 = DC-charge voltage HSB (hex/10 V)
  B7 = AC-Mains freq (hex hz)

ID 0x3X6 8bytes
  B0 = Charger primary temp (Hex °C)
  B1 = Charger secondary temp (Hex °C)
  B2 = AC-Mains voltage LSB (Hex V)
  B3 = AC-Mains voltage HSB (Hex V)
  B4 = Max Power LSB (hex W)
  B5 = Max Power HSB (hex W)
  B6 = Available Power (hex/2 %)
  B7 = 00

ID 0x3X6 

Watchdog
https://forum.arduino.cc/t/watchdog-timer-arm-arduino-due-board-ide-1-8-9/663311
https://forum.arduino.cc/t/arduino-due-watchdog/486187

rstc_start_software_reset(RSTC); //DUE reset command   
  
*/
/********************************************************************************
  extern "C" void _watchdogDefaultSetup (void) { WDT_Disable (WDT); }

  void watchdogSetup (void) __attribute__ ((weak, alias("_watchdogDefaultSetup")));
*********************************************************************************/

/*This function is called from init(). If the user does not provide
  this function, then the default action is to disable watchdog.
  This function has to be overriden, otherwise watchdog won't work !! */

#include <due_can.h>  //https://github.com/collin80/due_can
#include <due_wire.h> //https://github.com/collin80/due_wire
#include <DueTimer.h>  //https://github.com/collin80/DueTimer
#include <Wire_EEPROM.h> //https://github.com/collin80/Wire_EEPROM
#include <DuePWM.h>

#define WDT_KEY (0xA5)
//These two lines redefine any serial calls to the native Serial port on the CANdue or EVTVDue and enables syntactic sugar to allow streaming to serial port Serial<<"Text";

#define Serial SerialUSB
template<class T> inline Print &operator <<(Print &obj, T arg) { obj.print(arg); return obj; } //Sets up serial streaming Serial<<someshit;

 // Useful macros for setting and resetting bits
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#define PWM_FREQ1  120
#define PWM_FREQ2  130
DuePWM pwm( PWM_FREQ1, PWM_FREQ2 );

// PP pin from EVSE
int PP_pin = 36; // input connected to pin 36
int CP_pin = 44; // CP signal
int PP_relay = 30; // relay output connected to pin 30

// DCDC pin Enable
int Enable_pin = 38; // Enable input connected to pin 38
int DCDC_active = 46; // relay output connected to pin 46

// BMS pin input
int BMS_pin = 42; // BMS input connected to pin 42

//Heater and AC compressor command 
int Heater_pin = A4; // BMS input connected to pin A4
int AC_pin = A2; // BMS input connected to pin A3

//The Pin your button is attached to
const byte button = 32;
byte buttonState = 0;
byte lastButtonState = HIGH;
uint8_t buttonCounter = 0;
int LED1_pin = 28;
int LED2_pin = 45;
int LED3_pin = 43;

//*********GENERAL VARIABLE   DATA ******************

CAN_FRAME outframe;  //A structured variable according to due_can library for transmitting CAN data.
CAN_FRAME gearFrame; //We want a separate one for msg 228 as we use a timer interrupt to send this one every 10 ms

//Other ordinary variables
float Version=1.30;
uint16_t page=300;    //EEPROM page to hold variable data.  We save time by setting up a structure and saving a large block
int i;
unsigned long elapsedtime,time228,timestamp,startime,lastime;//Variables to compare millis for timers
boolean debug=false;
uint8_t logcycle=0;
uint8_t voltage = 0;
uint8_t Ctemp0 = 0;
uint8_t Ctemp1 = 0;
uint8_t Ctemp2 = 0;
uint8_t Cstat0 = 0;
uint8_t Cstat1 = 0;
uint8_t Cstat2 = 0;
uint8_t Htemp = 0;
uint8_t Heatertemp = 0;
uint8_t ACrpm1 = 0;
uint8_t ACrpm2 = 0;
uint8_t ACrpm = 0;

//Charger flags
bool Charger1 = true; //flag for charger1
bool Charger2 = true; //flag for charger2
bool Charger3 = true; //flag for charger3
//******* END OF GENERAL VARIABLE DATA***********






//*********EEPROM  DATA ******************
/*
 * This section provides a data struction and function for reading and writing EEPROM data containing persistent configuration variable we want to retain from 
 * one session to the next.  ANy kind of legal variable can be added to the structure and periodically during main loop execution we will write all variable data as a block\
 * to the EEPROM.  This minimizes the time delay hit of EEPROM writes.  You can update these variables as often as you like in other sections of the program and 
 * it will be actually saved to EEPROM during the next block write.
 * 
 * The initialize function is normally executed on new hardware where no previously written EEPROM data is found.  You can set default values here and on the first program 
 * execution those values will be loaded and used by the program.  Subsequent changes will be written to EEPROM and during the next session, the updated variables will be 
 * retrieved from EEPROM.
 */

//Store any configuration variables or other data you want to be persistent between power cycles.  Available in myVars.variable

class EEPROMvariables {
  public:
    uint8_t CANport;
    uint32_t datarate;
    uint16_t transmitime;
    uint8_t varsGood;
    uint8_t buttonCounter;
};
EEPROMvariables myVars;

void initializeEEPROMPage()
{
  //If no EEPROM data can be found at the EEPROM page, initialize the variables held in myVars here.  It will then be written to the designated EEPROM page
  
  myVars.CANport=0;
  myVars.datarate=500;
  myVars.transmitime=100;
  myVars.varsGood=20;
  myVars.buttonCounter=4;
  EEPROM.write(page,myVars);

}

//*********EEPROM  DATA ******************

/*This function is called from init(). If the user does not provide
  this function, then the default action is to disable watchdog.
  This function has to be overriden, otherwise watchdog won't work !! */

void watchdogSetup(void) {/*** watchdogDisable (); ***/}

//void watchdogDefaultSetup (void) { }
//void watchdogSetup(void) { watchdogDefaultSetup ();}

//********************SETUP FUNCTION*******I*********
/*
 * The SETUP function in the Arduino IDE simply lists items that must be performed prior to entering the main program loop.  In this case we initialize Serial 
 * communications via the USB port, set an interrupt timer for urgent outbound frames, zero our other timers, and load our EEPROM configuration data saved during the 
 * previous session.  If no EEPROM data is found, we initialize EEPROM data to default values
 * 
 */
void setup() 
  {
  // Enable watchdog.
  WDT->WDT_MR = WDT_MR_WDD(0xFFF)
                | WDT_MR_WDRPROC
                | WDT_MR_WDRSTEN
                | WDT_MR_WDV(256 * 2); // Watchdog triggers a reset after 2 seconds if underflow
                                       // 2 seconds equal 84000000 * 2 = 168000000 clock cycles
  /* Slow clock is running at 32.768 kHz
    watchdog frequency is therefore 32768 / 128 = 256 Hz
    WDV holds the periode in 256 th of seconds  */
   
    Serial.begin(115200);  //Initialize our USB port which will always be redefined as SerialUSB to use the Native USB port tied directly to the SAM3X processor.
    Timer4.attachInterrupt(sendCANframeURGENT).start(10000); // This sets an interrupt time to send an URGENT CAN frame every 10ms.  The function can be changed
                                                             // and the time can be changed.
    
    lastime=startime=time228=timestamp=millis();  //Zero our other timers
    elapsedtime=millis();
    
    
 //Load/validate EEPROM variable
    Wire.begin();
    EEPROM.setWPPin(19);     
    EEPROM.read(page, myVars);
    if(myVars.varsGood!=20)initializeEEPROMPage(); //We check to see if we have a saved page from earlier session. If not, initialize EEPROM variables and save.
 
//  Initialize CAN ports 
    if(myVars.CANport==0) initializeCAN(0);     
       else initializeCAN(1); 

  Can0.begin(CAN_BPS_500K); //directly declare CAN0 at 500K

 //Print welcome screen and menu  
    Serial<<"\n\n Startup successful. EVTV Motor Werks -  Program Name Version "<<Version<<"\n\n";
    printMenu(); 

uint32_t pwm_duty = 180; // 75% duty cycle
pwm.setFreq1( PWM_FREQ1 );
pwm.setFreq2( PWM_FREQ2 );
    // Setup PWM Once (Up to two unique frequencies allowed
    //-----------------------------------------------------  
pwm.pinFreq1( 6 );  // Pin 6 freq set to "pwm_freq1" on clock A
pwm.pinFreq1( 7 );  // Pin 7 freq set to "pwm_freq2" on clock B

    // Write PWM Duty Cycle Anytime After PWM Setup
    //-----------------------------------------------------    
pwm.pinDuty( 6, pwm_duty );  // 50% duty cycle on Pin 6
pwm.pinDuty( 7, pwm_duty );  // 50% duty cycle on Pin 7

pwm.stop( 6 );
pwm.stop( 7 );

pinMode(PP_pin,INPUT_PULLUP); // set PP pin to input with using built in pull up resistor
pinMode(PP_relay,OUTPUT); // 
pinMode(Enable_pin,INPUT); // 
pinMode(DCDC_active,OUTPUT); 
pinMode(BMS_pin,INPUT_PULLUP); //BMS HIGH PIN
pinMode(Heater_pin,INPUT_PULLUP); // set Heater pin to input with using built in pull up resistor
pinMode(AC_pin,INPUT_PULLUP); // set AC compressor pin to input with using built in pull up resistor
pinMode(button, INPUT_PULLUP);
pinMode(LED1_pin,OUTPUT); // LED1
pinMode(LED2_pin,OUTPUT); // LED2
pinMode(LED3_pin,OUTPUT); // LED3
Can0.watchFor();  

}
   
//********************END SETUP FUNCTION*******I*********





//********************MAIN PROGRAM LOOP*******I*********
/*
 * This is the main program loop. Lacking interrupts, this loop is simply repeated endlessly.  Show is an example of code that only executes after a certain
 * amount of time has passed. myVars.transmitime.  It also uses a counter to perform a very low priority function every so many loops.
 * Any other entries will be performed with each loop and on Arduino Due, this can be very fast.
 */



void loop()
{ 

 //Restart watchdog
  WDT->WDT_CR = WDT_CR_KEY(WDT_KEY)
                | WDT_CR_WDRSTT;

    /* If the software becomes trapped in a deadlock,
       watchdog triggers a reset and software restarts with stored values
       in General Purpose Back up Registers*/
  
 CAN_FRAME incoming;
 
  if (Can0.available() > 0) {
    Can0.read(incoming);
    if(incoming.id==0x305){
      Cstat0 = incoming.data.bytes[0] ;
      }
    if(incoming.id==0x315){
      Cstat1 = incoming.data.bytes[0] ;
      }
    if(incoming.id==0x325){
      Cstat2 = incoming.data.bytes[0] ;
      }           
    if(incoming.id==0x306){
      Ctemp0 = incoming.data.bytes[0] ;
      } 
    if(incoming.id==0x316){
      Ctemp1 = incoming.data.bytes[0] ;
    }
    if(incoming.id==0x326){
      Ctemp2 = incoming.data.bytes[0] ;
     // auxvoltage=((aux1 * 256) + aux2); //recalculate two bit voltage value
    }
    if(incoming.id==0x398){
      Htemp = incoming.data.bytes[3] ;
      Heatertemp = Htemp-40;
    } 
    if(incoming.id==0x388){
      ACrpm1 = incoming.data.bytes[2] ;
      ACrpm2 = incoming.data.bytes[3] ;      
      ACrpm = ((ACrpm1 * 256) + ACrpm2); //recalculate two bit voltage value
    }               
    }

checkButton();
if(digitalRead(PP_pin) == LOW) { // if PP_pin senses EVSE
digitalWrite(PP_relay,HIGH); // turn on Cp_relay
  if(millis()-lastime > myVars.transmitime)  //Nominally set for 120ms - do stuff on 120 ms non-interrupt clock
    {
     if(Charger1 == true){
     lastime=millis();        
      sendCANframeA();  
      printstatus();
digitalWrite(LED1_pin, HIGH);
         } 
         else {
digitalWrite(LED1_pin, LOW);
 }      
     if(Charger2 == true){
     lastime=millis();        
      sendCANframeB();
      printstatus();
digitalWrite(LED2_pin, HIGH);       
    }
         else {
digitalWrite(LED2_pin, LOW);
 }  
     if(Charger3 == true){
     lastime=millis();        
      sendCANframeC();
      printstatus();
digitalWrite(LED3_pin, HIGH);       
}
         else {
digitalWrite(LED3_pin, LOW);
 }         
}      
}             
else {
digitalWrite(PP_relay,LOW); // turn off PP_relay
digitalWrite(LED1_pin, LOW);
digitalWrite(LED2_pin, LOW);
digitalWrite(LED3_pin, LOW);
 } 
    
if (digitalRead(Enable_pin) == HIGH) { // if Enable_pin senses ignition 
  if(millis()-lastime > myVars.transmitime)  //Nominally set for 120ms - do stuff on 120 ms non-interrupt clock
    {   
     lastime=millis();        //Zero our timer
       sendCANframeE(); 
       printstatus(); 
       sendCANframeD();
       printstatus(); 
       }         
digitalWrite(DCDC_active,HIGH); // turn on DCDC_active relay
PWMsignalDCDC(); //signal DCDC ON
} 

else if(digitalRead(PP_relay) == HIGH) { // If charger is connected
digitalWrite(DCDC_active,LOW); // turn off DCDC_active relay   
pwm.stop( 6 );
 } 
 
else {
digitalWrite(DCDC_active,LOW); // turn off DCDC_active relay
PWMsignalDCDCoff(); //duty at 33% to shut down DCDC
 }           

 
     if(logcycle++ > 200) //Every 30 seconds, save configuration and data to EEPROM
       {  
        logcycle=0;
        Timer4.stop();    //Turn off the 10ms interrupt for 228 to avoid conflicts
        EEPROM.write(page, myVars);
        Timer4.start();    
        }         
}
//********************END MAIN PROGRAM LOOP*******I*********






  
 //********************USB SERIAL INPUT FROM KEYBOARD *******I*********
 /* These routines use an automatic interrupt generated by SERIALEVENT to process keyboard input any time a string terminated with carriage return is received 
  *  from an ASCII terminal program via the USB port.  The normal program execution is interrupted and the checkforinput routine is used to examine the incoming
  *  string and act on any recognized text.
  * 
  */
   
void serialEventRun(void) 
{
   if (Serial.available())checkforinput(); //If serial event interrupt is on USB port, go check for keyboard input           
}

void PWMsignalDCDC()
{
pwm.setFreq1( 120 ); //setup frequency
pwm.pinFreq1( 6 );  // Pin set to "pwm_freq1" on clock A  
pwm.pinDuty( 6, 180 );  // duty cycle on Pin 
}

void PWMsignalDCDCoff()
{
pwm.setFreq1( 120 ); //setup frequency
pwm.pinFreq1( 6 );  // Pin set to "pwm_freq1" on clock A  
pwm.pinDuty( 6, 85 );  // duty cycle on Pin 
}

void checkButton()
{
  buttonState = digitalRead(button);
  if (lastButtonState != buttonState) //if there is a change in button state
  {
    lastButtonState = buttonState; //has the button been released (HIGH is not pushed)
    if (buttonState == HIGH)
    {
      buttonCounter++; //button counter counts ahead
      
      if (buttonCounter == 1) { //first charger selected
        Charger1 = true;        
        Charger2 = false;
        Charger3 = false;    
      }
      else if (buttonCounter == 2) {
        Charger1 = false;        
        Charger2 = true;
        Charger3 = false;      
      }
      else if (buttonCounter == 3) {
        Charger1 = false;        
        Charger2 = false;
        Charger3 = true;
      }
      else if (buttonCounter == 4) {
        Charger1 = true;        
        Charger2 = true;
        Charger3 = false;
      }
      else if (buttonCounter == 5) {
        Charger1 = false;        
        Charger2 = true;
        Charger3 = true;
      }
      else if (buttonCounter == 6) 
      {
        Charger1 = true;        
        Charger2 = false;
        Charger3 = true;
      }
      else
      {
        buttonCounter = 0;
        Charger1 = true;        
        Charger2 = true;
        Charger3 = true;  
      }         
  }       
}
} 

/*
void chargertempctrl()
{
if (Ctemp >= 105) { // if Ctemp is high
digitalWrite(DCDC_active,HIGH); // turn off DCDC_active relay 
} 
else { // if Ctemp is lower 
digitalWrite(DCDC_active,LOW); // turn off DCDC_active relay
}}
*/

void checkforinput()
{
  //Checks for input from Serial Port 1 indicating configuration items most likely.  Handle each case.
  
   if (Serial.available()) 
   {
    int inByte = Serial.read();
    switch (inByte)
     {
    	 case '?':
      		printMenu();
      		break; 	
       case 'h':
      		printMenu();
      		break;
     	 case 'D':
      		debug=(!debug);
      		break;
	     case 'd':
      		debug=(!debug);
      		break;
     
       case 'i':
      		getInterval();
      		break;
       case 'I':
      		getInterval();
      		break;
	     case 'c':
      		getPort();
      		break;
       case 'C':
      		getPort();
      		break;
       case 'k':
      		getRate();
      		break;
       case 'K':
      		getRate();
      		break;
      
   	  }    
    }
}





void getInterval()
{
	Serial<<"\n Enter the interval in ms between each CAN frame transmission : ";
		while(Serial.available() == 0){}               
		float V = Serial.parseFloat();	
		if(V>0)
		  {
        Serial<<V<<"CAN frame interval\n\n";
        myVars.transmitime=abs(V);
      }
}
       

void getRate()
{
	Serial<<"\n Enter the Data Rate in Kbps you want for CAN : ";
		while(Serial.available() == 0){}               
		float V = Serial.parseFloat();	
		if(V>0)
		  {
       Serial<<V<<"\n\n";
       myVars.datarate=V*1000;
       initializeCAN(myVars.CANport);
		  }
}


void getPort()
{
	Serial<<"\n Enter port selection:  c0=CAN0 c1=CAN1 ";
	while(Serial.available() == 0){}               
	int P = Serial.parseInt();	
	myVars.CANport=P;
	if(myVars.CANport>1) Serial<<"Entry out of range, enter 0 or 1 \n";
  else 
     {
     if(myVars.CANport==0) initializeCAN(0); //If CANdo is 0, initialize CAN0 
     if(myVars.CANport==1) initializeCAN(1);    //If CANdo is 1, initialize CAN1
   	 }           
}

//********************END USB SERIAL INPUT FROM KEYBOARD ****************









//******************** USB SERIAL OUTPUT TO SCREEN ****************
/*  These functions are used to send data out the USB port for display on an ASCII terminal screen.  Menus, received frames, or variable status for example
 *   
 */

void printMenu()
{
  Serial<<"\f\n=========== Tesla Model S CAN Controller  Version "<<Version<<" ==============\n************ List of Available Commands ************\n\n";
  Serial<<"  ? or h  - Print this menu\n";
  Serial<<"  c - sets CAN port ie c0 or c1\n";
  Serial<<"  d - toggles debug DISPLAY FRAMES to print CAN data traffic\n";
  Serial<<"  i - set interval in ms between CAN frames i550 \n";
  Serial<<"  k - set data rate in kbps ie k500 \n";
 
 Serial<<"**************************************************************\n==============================================================\n\n"; 
}

void printFrame(CAN_FRAME *frame,int sent)
{ 
  char buffer[300];
  sprintf(buffer,"msgID 0x%03X; %02X; %02X; %02X; %02X; %02X; %02X; %02X; %02X  %02d:%02d:%02d.%04d\n", frame->id, frame->data.bytes[0], 
  frame->data.bytes[1],frame->data.bytes[2], frame->data.bytes[3], frame->data.bytes[4], frame->data.bytes[5], frame->data.bytes[6],
  frame->data.bytes[7], hours(), minutes(), seconds(), milliseconds());
  
   if(sent)Serial<<"Sent ";
    else Serial<<"Received ";       
   Serial<<buffer<<"\n";
}

void printstatus()
{
  /* This function prints an ASCII statement out the SerialUSB port summarizing various data from program variables.  Typically, these variables are updated
   *  by received CAN messages that operate from interrupt routines. This routine also time stamps the moment at which it prints out.
   */
  
    char buffer[300]; 
  //  sprintf(buffer,"CAN%i Int:%ims Regen:%i%% Throt:%i%% RPM:%i MPH:%i Torq %iNm %3.1fvdc %iAmps %3.1fkW MotT1:%iC InvT5:%iC  %02d:%02d:%02d.%04d\n\n",myVars.CANport,
//    myVars.transmitime,regenpercent,throttle,rpm,mph, torq1,voltage,current,power,temperature1,temperature5,hours(),minutes(),seconds(),milliseconds());
 
      sprintf(buffer,"CAN%i Rate:%i Int:%ims  %02d:%02d:%02d.%04d\n\n",myVars.CANport, myVars.datarate, myVars.buttonCounter, myVars.transmitime,hours(),minutes(),seconds(),milliseconds());
 
    Serial<<buffer;
}


int milliseconds(void)
{
  int milliseconds = (int) (micros()/100) %10000 ;
  return milliseconds;
}


 int seconds(void)
{
    int seconds = (int) (micros() / 1000000) % 60 ;
    return seconds;
}


int minutes(void)
{
    int minutes = (int) ((micros() / (1000000*60)) % 60);
    return minutes;
}

    
int hours(void)
{    
    int hours   = (int) ((micros() / (1000000*60*60)) % 24);
    return hours;
}  



//******************** END USB SERIAL OUTPUT TO SCREEN ****************






//******************** CAN ROUTINES ****************************************
/* This section contains CAN routines to send and receive messages over the CAN bus
 *  INITIALIZATION routines set up CAN and are called from program SETUP to establish CAN communications.
 *  These initialization routines allow you to set filters and interrupts.  On RECEIPT of a CAN frame, an interrupt stops execution of the main program and 
 *  sends the frame to the specific routine used to process that frame by Message ID. Once processed, the main program is resumed.
 *  
 *  Frames can also be sent, either from the main control loop, or by a Timer interrupt allowing specific frames to be sent on a regular interrupt interval.
 *  
 *  For example a frame that MUST go out every 10 ms would use the Timer Interrupt in SETUP to cause a CAN function here to send a frame every 10 ms.  Other frames 
 *  could be sent normally from the main program loop or conditionally based on some received or calculated data.
 *  
 */

void initializeCAN(int which)
{
  //Initialize CAN bus 0 or 1 and set filters to capture incoming CAN frames and route to interrupt service routines in our program.
  
  if(which)  //If 1, initialize Can1.  If 0, initialize Can0.
    {
      pinMode(48,OUTPUT);
      if (Can1.begin(myVars.datarate,48)) 
        {
          Serial.println("Using CAN1 - initialization completed.\n");
          Can1.setNumTXBoxes(4);
          Can1.setRXFilter(0, 0x106, 0x7FF, false);
          Can1.setCallback(0, handleCANframe);
          Can1.setRXFilter(1, 0x618, 0x7FF, false);
          Can1.setCallback(1, handleCANframe);
          Can1.setRXFilter(2, 0x274, 0x7FF, false);
          Can1.setCallback(2, handleCANframe);
          Can1.setRXFilter(3, 0x1D6, 0x7FF, false);
          Can1.setCallback(3, handleCANframe);
          Can1.setRXFilter(4, 0x30A, 0x7FF, false);
          Can1.setCallback(4, handleCANframe);
          Can1.setRXFilter(5, 0x308, 0x7ff, false);
          Can1.setCallback(5, handleCANframe);
          Can1.setRXFilter(6, 0x268, 0x700, false);
          Can1.setCallback(6, handleCANframe);           
        }
           else Serial.println("CAN1 initialization (sync) ERROR\n");
    } 
  else
    {//else Initialize CAN0
     pinMode(50,OUTPUT);
     if (Can0.begin(myVars.datarate,50)) 
        {
          Serial.println("Using CAN0 - initialization completed.\n");
          Can0.setNumTXBoxes(4);
          Can0.setRXFilter(0, 0x2FF, 0x3FF, false);
          Can0.setCallback(0, handleCANframe);
          Can0.setRXFilter(1, 0x2FF, 0x3FF, false);
          Can0.setCallback(1, handleCANframe); 
          Can0.setRXFilter(2, 0x2FF, 0x3FF, false);
          Can0.setCallback(2, handleCANframe);
          Can0.setRXFilter(3, 0x2FF, 0x3FF, false);
          Can0.setCallback(3, handleCANframe);                       
          Can0.setRXFilter(4, 0x2FF, 0x3FF, false);
          Can0.setCallback(4, handleCANframe);
          }
        else Serial.println("CAN0 initialization (sync) ERROR\n");
    }
}   

void handleCANframe(CAN_FRAME *frame)
//This routine handles CAN interrupts from a capture of any other CAN frames from the inverter not specifically targeted.  
//If you add other specific frames, do a setRXFilter and CallBack mailbox for each and do a method to catch those interrupts after the fashion
//of this one.
{  
    //This routine basically just prints the general frame received IF debug is set.  Beyond that it does nothing.
    
    if(debug) printFrame(frame,0); //If DEBUG variable is 1, print the actual message frame with a time stamp showing the time received.      
}

void handle389frame(CAN_FRAME *frame)
//This routine handles CAN interrupts from  Address 0x106 CAN frame   
{   
int voltage;  
int Ctemp;
//  float throttle;
//  int rpm;
//  int wheelrpm;
//  int torq2;
//  int torq1;
voltage=word(frame->data.bytes[0]*2); 
Ctemp0=word(frame->data.bytes[4]);  
//    throttle=round(frame->data.bytes[6]/2.56);
//    rpm=word(frame->data.bytes[5],frame->data.bytes[4]);
//    wheelrpm=rpm/9.73;
//    torq2=(word(frame->data.bytes[3],frame->data.bytes[2]))&8191;
//      if(torq2>4095)torq2=(8192-torq2)*-1;
//      torq2/=10;
//    torq1=(word(frame->data.bytes[1],frame->data.bytes[0]))&8191;
//      if(torq1>4095)torq1=(8192-torq1)*-1;
//     torq1/=10;
      
    if(debug)printFrame(frame,0); //If DEBUG variable is 1, print the actual message frame with a time stamp showing the time received.    
}

// 0x377h 8bytes DC-DC converter status
//  B0+B1 = 12V Battery voltage (h04DC=12,45V -> 0,01V/bit)
//  B2+B3 = 12V Supply current  (H53=8,3A -> 0,1A/bit)
//void handle377frame(CAN_FRAME *frame)
////This routine handles CAN interrupts from  Address 0x377 CAN frame   
//{    
//float auxvoltage; 
//float auxcurrent;
//auxvoltage=word((frame->data.bytes[0], frame->data.bytes[1])); // word(Highbyte, Lowbyte)
//auxcurrent=word((frame->data.bytes[2], frame->data.bytes[3])); // word(Highbyte, Lowbyte)
//int combined = ((int) highByte << 8) | lowByte;

//}

void sendCANframeA() {
        outframe.id = 0x300;            // Set our transmission address ID
        outframe.length = 8;            // Data payload 8 bytes
        outframe.extended = 0;          // Extended addresses - 0=11-bit 1=29bit
        outframe.rtr=1;                 //No request
        outframe.data.bytes[0]=0x01;
        outframe.data.bytes[1]=0xE8;  
        outframe.data.bytes[2]=0x03; 
        outframe.data.bytes[3]=0x54; // 418V
        outframe.data.bytes[4]=0x10;
        if(digitalRead(BMS_pin) == HIGH) { // if BMS_pin is high
        outframe.data.bytes[5]=0x78; // 78=12A -> 0,1A/bit
        } 
        else {
        outframe.data.bytes[5]=0x00;
        } 
        outframe.data.bytes[6]=0x00;
        outframe.data.bytes[7]=0x00;
        if(debug) {printFrame(&outframe,1);} //If the debug variable is set, show our transmitted frame
        if(myVars.CANport==0) Can0.sendFrame(outframe);    //Mail it
         else Can1.sendFrame(outframe); 
}

void sendCANframeB() {
       outframe.id = 0x310;            // Set our transmission address ID
        outframe.length = 8;            // Data payload 8 bytes
        outframe.extended = 0;          // Extended addresses - 0=11-bit 1=29bit
        outframe.rtr=1;                 //No request
        outframe.data.bytes[0]=0x01;
        outframe.data.bytes[1]=0xE8;  
        outframe.data.bytes[2]=0x03; 
        outframe.data.bytes[3]=0x54; // 418V
        outframe.data.bytes[4]=0x10;
        if(digitalRead(BMS_pin) == HIGH) { // if BMS_pin is high
        outframe.data.bytes[5]=0x78; // 78=12A -> 0,1A/bit
        } 
        else {
        outframe.data.bytes[5]=0x00;
        }  
        outframe.data.bytes[6]=0x00;
        outframe.data.bytes[7]=0x00;
       if(debug) {printFrame(&outframe,1);} //If the debug variable is set, show our transmitted frame             
       if(myVars.CANport==0) Can0.sendFrame(outframe);    //Mail it
         else Can1.sendFrame(outframe); 
}

void sendCANframeC() {
       outframe.id = 0x320;            // Set our transmission address ID
        outframe.length = 8;            // Data payload 8 bytes
        outframe.extended = 0;          // Extended addresses - 0=11-bit 1=29bit
        outframe.rtr=1;                 //No request
        outframe.data.bytes[0]=0x01;
        outframe.data.bytes[1]=0xE8;  
        outframe.data.bytes[2]=0x03; 
        outframe.data.bytes[3]=0x54; // 418V
        outframe.data.bytes[4]=0x10;
        if(digitalRead(BMS_pin) == HIGH) { // if BMS_pin is high
        outframe.data.bytes[5]=0x78; // 78=12A -> 0,1A/bit
        } 
        else {
        outframe.data.bytes[5]=0x00;
        } 
        outframe.data.bytes[6]=0x00;
        outframe.data.bytes[7]=0x00;
       if(debug) {printFrame(&outframe,1);} //If the debug variable is set, show our transmitted frame             
       if(myVars.CANport==0) Can0.sendFrame(outframe);    //Mail it
         else Can1.sendFrame(outframe); 
}

void sendCANframeD() { // Heater
       outframe.id = 0x188;            // 0x188 03 50 A2 40 00 00 00 00 
        outframe.length = 8;            // Data payload 8 bytes
        outframe.extended = 0;          // Extended addresses - 0=11-bit 1=29bit
        outframe.rtr=1;                 //No request
        if(digitalRead(Heater_pin) == LOW) { // if heater pin is ON 
        outframe.data.bytes[0]=0x03; // when 55deg power goes to 0A
        }     
        else {
        outframe.data.bytes[0]=0x00;
        }          
        outframe.data.bytes[1]=0x50; // 50 works
        if(Heatertemp > 55) { // if temp is higher than 55deg  
        outframe.data.bytes[2]=0x00; // when 55deg power goes to 0A
        } 
        else if(Heatertemp > 50) { // if temp is higher than 50deg
        outframe.data.bytes[2]=0x32; // seems to be current command (dec/10)
        }         
        else {
        outframe.data.bytes[2]=0xA2;
        }         
        outframe.data.bytes[3]=0x40; // 40 works 
        outframe.data.bytes[4]=0x00; //
        outframe.data.bytes[5]=0x00; // 
        outframe.data.bytes[6]=0x00;
        outframe.data.bytes[7]=0x00;
       if(debug) {printFrame(&outframe,1);} //If the debug variable is set, show our transmitted frame             
       if(myVars.CANport==0) Can0.sendFrame(outframe);    //Mail it
         else Can1.sendFrame(outframe); 
}

void sendCANframeE() { //AC compressor
       outframe.id = 0x185;            // 0x185 0B 00 1D 00 00 08 00 03 
        outframe.length = 8;            // Data payload 8 bytes
        outframe.extended = 0;          // Extended addresses - 0=11-bit 1=29bit
        outframe.rtr=1;                 //No request 
        if(digitalRead(AC_pin) == LOW) { //AC pin is active        
        outframe.data.bytes[0]=0x0B; // status command 08 stbd, 0B start
        }   
        else {
        outframe.data.bytes[0]=0x08; //if we sense LOW we go to standby
        }              
        outframe.data.bytes[1]=0x00; // 
        outframe.data.bytes[2]=0x1D; // 1D works
        outframe.data.bytes[3]=0x00; //
        outframe.data.bytes[4]=0x00; //
        if(digitalRead(AC_pin) == LOW) { //AC pin is active
        outframe.data.bytes[5]=0x30; // RPM command 08 to 54
        }
        else {
        outframe.data.bytes[5]=0x08; //AC pin is inactive
        }        
        outframe.data.bytes[6]=0x00;
        outframe.data.bytes[7]=0x03;
       if(debug) {printFrame(&outframe,1);} //If the debug variable is set, show our transmitted frame             
       if(myVars.CANport==0) Can0.sendFrame(outframe);    //Mail it
         else Can1.sendFrame(outframe); 
}

void sendCANframeURGENT() {
        outframe.id = 0x285;            // 0x285 00,00,14,39,91,FE,0C,10 Driving
        outframe.length = 8;            // Data payload 8 bytes
        outframe.extended = 0;          // Extended addresses - 0=11-bit 1=29bit
        outframe.rtr=1;                 //No request
        outframe.data.bytes[0]=0x00; 
        outframe.data.bytes[1]=0x00;
        if(digitalRead(PP_pin) == HIGH) { // we are driving, no EVSE connected
        outframe.data.bytes[2]=0x14; // PP EVSE connected signal
        }   
        else {
        outframe.data.bytes[2]=0xB6; //if we sense PP signal we are charging
        }      
        outframe.data.bytes[3]=0x39;
        outframe.data.bytes[4]=0x91;
        outframe.data.bytes[5]=0xFE;
        outframe.data.bytes[6]=0x0E;
        outframe.data.bytes[7]=0x10;
        if(debug) {printFrame(&outframe,1);} //If the debug variable is set, show our transmitted frame
        if(myVars.CANport==0) Can0.sendFrame(outframe);    //Mail it
         else Can1.sendFrame(outframe); 
}

//******************** END CAN ROUTINES ****************
